@@ -60,6 +60,17 @@ class LatticeAdaptor:
         self.history.put((deepcopy(self.name), deepcopy(self.len), deepcopy(self.table)))
 
         self._table = value
+        if value is not None:
+            self.builder.definitions = (
+                self.table.drop(["at", "pos"], axis=1, errors="ignore")
+                .drop_duplicates()
+                .set_index("name", drop=True)
+                .to_dict(orient="index")
+            )
+            self.builder.positions = self.table[["name", "at"]]
+            self.builder.lattice = list(self.table["name"].values)
+            self.builder.name = self.name
+            self.update_table()
 
     def load_from_file(self, filename: str, ftype: str = "lte") -> None:
         """Load data from file.
@@ -83,7 +94,7 @@ class LatticeAdaptor:
         except Exception:
             self.name = "ring"
             print("Lattice name has been autoset - check if value is ok.")
-        self.table = self.builder.table
+        self.table = self.builder.table.fillna(0.0)
 
         # length is last element center pos + half the  length
         with open(filename, "r") as f:
@@ -262,7 +273,7 @@ class LatticeAdaptor:
                 #    nextrow["pos"] > row.pos,
                 #    nextrow["pos"] - (nextrow["L"] / 2.0) > row.pos + row.L / 2.0,
                 # )
-                if nextrow["pos"] - (nextrow["L"] / 2.0) > row.pos + row.L / 2.0:
+                if nextrow["pos"] - (nextrow["L"] / 2.0) > (row.pos + row.L / 2.0 - 1.0e-6):
                     ndrift += 1
                     newrow = {}
                     newrow["name"] = name + str(ndrift)
@@ -283,6 +294,11 @@ class LatticeAdaptor:
             newrows.append(pd.Series(newrow).to_frame().T)
 
         self.table = (pd.concat(newrows)).reset_index(drop=True)
+        print(self.table.columns)
+        print(pd.concat(newrows).columns)
+
+        # in order to correct missing values in 'at'
+        self.table["at"] = self.table["pos"]
 
     def parse_table_to_madx_line_string(self) -> str:
         """Method to convert the table to a MADX line definition lattice.
@@ -411,12 +427,12 @@ class LatticeAdaptor:
         # updated history
         self.history.put((deepcopy(self.name), deepcopy(self.len), deepcopy(self.table)))
 
-        self.builder.positions = None
+        # self.builder.positions = None
         self.builder.build_table()
 
         # extract the relavant info
         self.name = self.builder.name
-        self.table = self.builder.table
+        self._table = self.builder.table
 
         # length is last element center pos + half the  length
         print("Length has been autoset - check if value is ok - otherwise update it.")
